@@ -7,7 +7,7 @@ from guietta import Gui, _, R1, R2, R3
 
 gui = Gui(
 
-  [ R1('rad1')  ,  R2('rad3')  , R3('rad5')   ],
+  [ R1('rad1',1),  R2('rad3')  , R3('rad5')   ],
   [ R1('rad2')  ,  R2('rad4')  , R3('rad6')   ],
 
 )
@@ -33,4 +33,4 @@ with gui.rad1, gui.rad2:
 gui.rad3 = msg
 gui.rad4 = msg
 
-gui.run()
\ No newline at end of file
+gui.run()
