@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from guietta import B, E, _, Gui, Quit
-from PySide2.QtWidgets import QSlider
-from PySide2.QtCore import Qt
+from guietta import B, E, _, Gui, Quit, QSlider, Qt
 
 def do_eval(gui, *args):
     gui.result = eval(gui.expr)
