@@ -1,24 +1,19 @@
 # -*- coding: utf-8 -*-
 
-from PySide2.QtCore import Qt
-from guietta import Gui, _, ___, III, HValueSlider, VValueSlider
+from guietta import Gui, _, ___, III, HValueSlider, VValueSlider, Qt
 
 def do_slider(gui, value):
     print('Slider value: ', value)
     
-import numpy as np
-
 slider1 = HValueSlider('hvalue', range(500), unit='Hz')    
 slider2 = HValueSlider('hvalue', range(500), unit='Hz', anchor=Qt.AnchorLeft) 
-slider3 = HValueSlider('hvalue', range(500))    
-slider4 = VValueSlider('hvalue', range(500))    
+slider3 = VValueSlider('hvalue', range(500))    
 
 gui = Gui(
     
-  [  'xxx' , ['xxx'], 'xxx' , slider4],
+  [  'xxx' , ['xxx'], 'xxx' , slider3],
   [  slider1,   ___   , ___ ,   III  ],
   [  slider2,   ___   ,  _  ,   III  ],
-  [  slider3,   ___   ,  _  ,   III  ],
   )
 
 
@@ -26,7 +21,6 @@ gui.events(
     
   [     _     ,    _    ,   _  ,  _  ],
   [  do_slider,    _    ,   _  ,  _  ],
-  [     _     ,    _    ,   _  ,  _  ],
   [     _     ,    _    ,   _  ,  _  ],
    )
 
