@@ -11,6 +11,7 @@ List of widget shortcuts:
     B('text')      ->   same as ['text']
     B('image.jpg') ->   same as ['image.jpg']
     '__name__'     ->   QLineEdit(''), name set to 'name'
+    '__name__:text'->   QLineEdit('text'), name set to 'name'
     E('text')      ->   QLineEdit('text')
     C('text')      ->   QCheckBox('text')
     R('text')      ->   QRadioButton('text')
@@ -23,6 +24,10 @@ List of widget shortcuts:
     _              ->   QLabel('')
     ___            ->   (three underscores) Horizontal widget span
     III            ->   (three capital letters i) vertical widget span
+    M              ->   Matplotlib plot or image
+    MA             ->   Matplotlib plot or imag with fast update
+    PG             ->   pyqtgraph plot
+    PGI            ->   pyqtgraph image
 
     QPushButtons with both image and text:
     ['image.jpg', 'text']  ->   QPushButton(QIcon('image.jpg'), 'text')
@@ -40,6 +45,7 @@ Signals can be connected with gui.events() where every widget has:
 import re
 import ast
 import sys
+import time
 import queue
 import signal
 import inspect
@@ -55,27 +61,27 @@ from collections import namedtuple, defaultdict
 from collections.abc import Sequence, Mapping, MutableSequence
 
 try:
-    from PySide2.QtWidgets import QApplication, QLabel, QWidget, QAbstractSlider
-    from PySide2.QtWidgets import QPushButton, QRadioButton, QCheckBox, QFrame
-    from PySide2.QtWidgets import QLineEdit, QGridLayout, QSlider, QAbstractButton
-    from PySide2.QtWidgets import QMessageBox, QListWidget, QAbstractItemView
-    from PySide2.QtWidgets import QPlainTextEdit, QHBoxLayout, QComboBox
-    from PySide2.QtWidgets import QSplashScreen, QFileDialog, QButtonGroup
-    from PySide2.QtWidgets import QProgressBar, QGroupBox
-    from PySide2.QtGui import QPixmap, QIcon, QFont
-    from PySide2.QtCore import Qt, QTimer, Signal, QEvent
+    from PyQt5.QtWidgets import QApplication, QLabel, QWidget, QAbstractSlider
+    from PyQt5.QtWidgets import QPushButton, QRadioButton, QCheckBox, QFrame
+    from PyQt5.QtWidgets import QLineEdit, QGridLayout, QSlider, QAbstractButton
+    from PyQt5.QtWidgets import QMessageBox, QListWidget, QAbstractItemView
+    from PyQt5.QtWidgets import QPlainTextEdit, QHBoxLayout, QComboBox
+    from PyQt5.QtWidgets import QSplashScreen, QFileDialog, QButtonGroup
+    from PyQt5.QtWidgets import QProgressBar, QGroupBox
+    from PyQt5.QtGui import QPixmap, QIcon, QFont
+    from PyQt5.QtCore import Qt, QTimer, QEvent
+    from PyQt5.QtCore import pyqtSignal as Signal
 except ImportError:
     try:
-        from PyQt5.QtWidgets import QApplication, QLabel, QWidget, QAbstractSlider
-        from PyQt5.QtWidgets import QPushButton, QRadioButton, QCheckBox, QFrame
-        from PyQt5.QtWidgets import QLineEdit, QGridLayout, QSlider, QAbstractButton
-        from PyQt5.QtWidgets import QMessageBox, QListWidget, QAbstractItemView
-        from PyQt5.QtWidgets import QPlainTextEdit, QHBoxLayout, QComboBox
-        from PyQt5.QtWidgets import QSplashScreen, QFileDialog, QButtonGroup
-        from PyQt5.QtWidgets import QProgressBar, QGroupBox
-        from PyQt5.QtGui import QPixmap, QIcon, QFont
-        from PyQt5.QtCore import Qt, QTimer, QEvent
-        from PyQt5.QtCore import pyqtSignal as Signal
+        from PySide2.QtWidgets import QApplication, QLabel, QWidget, QAbstractSlider
+        from PySide2.QtWidgets import QPushButton, QRadioButton, QCheckBox, QFrame
+        from PySide2.QtWidgets import QLineEdit, QGridLayout, QSlider, QAbstractButton
+        from PySide2.QtWidgets import QMessageBox, QListWidget, QAbstractItemView
+        from PySide2.QtWidgets import QPlainTextEdit, QHBoxLayout, QComboBox
+        from PySide2.QtWidgets import QSplashScreen, QFileDialog, QButtonGroup
+        from PySide2.QtWidgets import QProgressBar, QGroupBox
+        from PySide2.QtGui import QPixmap, QIcon, QFont
+        from PySide2.QtCore import Qt, QTimer, Signal, QEvent
     except ImportError as e:
         raise Exception('At least one of PySide2 or PyQt5 must be installed') from e
 
@@ -150,7 +156,7 @@ class ContextMixIn():
         else:
             # Do not use inspect.getsourcelines because it appears
             # to fail on some systems
-            lines = open(self._start.filename).readlines()
+            lines = open(self._start.filename,encoding='utf-8').readlines()
             withlines = lines[self._start.lineno - 1 : end.lineno]
 
         withsource = textwrap.dedent(''.join(withlines))
@@ -323,6 +329,7 @@ def _alsoAcceptAnotherGui(widget):
                 else:
                     widget._gui.layout().replaceWidget(widget, value.window())
                     widget.hide()
+                widget._gui.add_as_subgui(value)
             else:
                 f(value)
         return wrapper
@@ -430,6 +437,20 @@ def _text_property(widget):
     return GuiettaProperty(get_text, set_text, widget)
 
 
+def _setonly_text_property(widget):
+    '''Text property that allows setting but not reading
+
+    The widget instance is returned when reading the property
+    '''
+
+    def get():
+        return widget
+
+    prop = _text_property(widget)
+    prop.get = get
+    return prop
+
+
 def _title_property(widget):
     '''Property for widgets with a title (QGroupBox)'''
 
@@ -518,18 +539,27 @@ class SmartQLabel(QWidget):
     and lists, which are shown one element per line.
     If a dict is passed, both labels are shown and used for keys and
     values respectively, one element per line.
+
+    If the initializator contains a % character, it is interpreted
+    as a format string that will be used as the format for later
+    assignments. This behaviour can be turned off by setting
+    gui.use_formats to False.
     '''
 
-    def __init__(self, text=''):
+    def __init__(self, gui, text=''):
 
         super().__init__()
+        self._mygui = gui
         self._layout = QHBoxLayout()
         self._left = QLabel('')
         self._right = QLabel('')
         self._layout.addWidget(self._left)
         self._layout.addWidget(self._right)
         self.setLayout(self._layout)
+        self._format = None
         self.setText(text)
+        if '%' in text:
+            self._format = text
 
     def setText(self, value):
 
@@ -547,7 +577,16 @@ class SmartQLabel(QWidget):
 
         else:
             self._right.hide()
-            self._left.setText(str(value))
+            if self._format and self._gui.use_formats:
+                mystr = self._format % value
+            else:
+                mystr = str(value)
+
+            fullpath, name = _image_fullpath(self._mygui, mystr)
+            if fullpath:
+                self._left.setPixmap(QPixmap(fullpath))
+            else:
+                self._left.setText(mystr)
 
         self._orig_value = value
 
@@ -558,6 +597,32 @@ class SmartQLabel(QWidget):
         return _text_property(self)
 
 
+class HeartBeat(SmartQLabel):
+    '''Heartbeat class
+
+       A label that changes appearance when triggered,
+       and reverts back to the original one after a delay.
+    '''
+    def __init__(self, gui, text_or_filename1, text_or_filename2):
+        self._text_or_filename1 = text_or_filename1
+        self._text_or_filename2 = text_or_filename2
+        super().__init__(gui, text_or_filename1)
+
+    def _endbeat(self, *args):
+        self.setText(self._text_or_filename1)
+
+    def _wait(self):
+        time.sleep(self._delay)
+
+    def beat(self, delay=0.2):
+        self._delay = delay
+        self.setText(self._text_or_filename2)
+        self._mygui.execute_in_background(self._wait, (), self._endbeat)
+
+    def __guietta_property__(self):
+        return _setonly_text_property(self)
+
+
 def _guietta_property(widget):
     '''Create the instance property corresponding to `widget`'''
 
@@ -596,6 +661,7 @@ class _DeferredCreationWidget:
     def create(self, gui):
         pass
 
+
 class HS(_DeferredCreationWidget):
     '''Horizontal slider'''
 
@@ -625,15 +691,29 @@ class _R(_DeferredCreationWidget):
 
     _group = None
 
-    def __init__(self, text):
+    def __init__(self, text, checked=False):
         self._text = text
+        self._checked = checked
 
     def create(self, gui):
         button = QRadioButton(self._text)
+        button.setChecked(self._checked)
         gui._groups[self._group].addButton(button)
         return (button, self._text)
 
 
+class HB(_DeferredCreationWidget):
+    '''Text label or image label'''
+
+    def __init__(self, text_or_filename1, text_or_filename2):
+        self._text_or_filename1 = text_or_filename1
+        self._text_or_filename2 = text_or_filename2
+
+    def create(self, gui):
+        fullpath, name = _image_fullpath(gui, self._text_or_filename1)
+        return (HeartBeat(gui, self._text_or_filename1, self._text_or_filename2), name)
+
+
 class R0(_R):
     '''Radio button in pre-defined button group #0'''
 
@@ -718,12 +798,7 @@ class L(_DeferredCreationWidget):
 
     def create(self, gui):
         fullpath, name = _image_fullpath(gui, self._text_or_filename)
-        if fullpath:
-            label = QLabel()
-            label.setPixmap(QPixmap(fullpath))
-            return (label, name)
-        else:
-            return (SmartQLabel(self._text_or_filename), name)
+        return (SmartQLabel(gui, self._text_or_filename), name)
 
 
 class B(_DeferredCreationWidget):
@@ -749,8 +824,11 @@ class _AutoConnectButton(_DeferredCreationWidget):
 
     def create(self, gui):
         button = QPushButton(self._text)
-        slot = getattr(gui, self._slot_name)
-        handler = _exception_wrapper(slot, gui._exception_mode)
+        if callable(self._slot_name):
+            slot = functools.partial(self._slot_name, gui)
+        else:
+            slot = getattr(gui, self._slot_name)
+        handler = _exception_wrapper(slot, gui)
         button.clicked.connect(handler)
         return button
 
@@ -878,18 +956,21 @@ class CB(_DeferredCreationWidget):
         return (cb, self._name)
 
 
+########################
+# Password entry field
+
+class PW(_DeferredCreationWidget):
+    '''A password entry field'''
+
+    def create(self, gui):
+        widget = QLineEdit('')
+        widget.setEchoMode(QLineEdit.Password)
+        return (widget, self._name)
+
+
 #################
 # Slider combined with editbox
 
-class _QLineEditDoNotErase(QLineEdit):
-    '''A QLineEdit that whose contents must not be erased
-
-    Gui constructor erases all the QLineEdit content because normally
-    it is the widget name. This class signals that the contents should
-    not be erased.
-    '''
-    pass
-
 
 class _CombinedWidget:
     '''Base class for widgets that combine multiple ones'''
@@ -924,7 +1005,7 @@ class _ValueSlider(_CombinedWidget):
         slider.setMaximum(stop)
         slider.setSingleStep(step)
 
-        editbox = _QLineEditDoNotErase()
+        editbox = QLineEdit('')
 
         def slider_to_unit(v):
             return v / factor
@@ -1062,33 +1143,44 @@ class Exceptions(Enum):
     pass                    # callable = custom exception handler
 
 
-def _exception_wrapper(func, mode):
+def _exception_handler(e, gui):
+    '''
+    Exception handler for all automatic actions and
+    signal slots. The exception moment is checked on the fly,
+    and thus can be changed at runtime.
+    '''
+    mode = gui._exception_mode
+
     if mode == Exceptions.OFF:
-        return func
+        raise e from None
 
     elif mode == Exceptions.SILENT:
-        handler = lambda e: None
+        pass
 
     elif mode == Exceptions.PRINT:
-        handler = lambda e: print('Exception: %s\n%s' %
-                                  (e.__class__.__name__, str(e)))
+        print('Exception: %s\n%s' % (e.__class__.__name__, str(e)))
 
     elif mode == Exceptions.POPUP:
-        handler = lambda e: QMessageBox.warning(None, "Error", "%s\n%s" %
-                                                (e.__class__.__name__, str(e)))
+        QMessageBox.warning(None, "Error", "%s\n%s" %
+                           (e.__class__.__name__, str(e)))
     elif callable(mode):
-        handler = mode
+        mode()
 
     else:
         raise TypeError('Exception mode must be either an instance of '
                         'the Exceptions enum or a callable handler.')
 
+def _exception_wrapper(func, gui):
+    '''
+    Wraps *func* with the default exception handler, based
+    on the exception mode set by *gui* at the moment of handling.
+    '''
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
         try:
             func(*args, **kwargs)
         except Exception as e:
-            handler(e)
+            _exception_handler(e, gui)
 
     return wrapper
 
@@ -1107,12 +1199,21 @@ def Ax(widget):
         with Ax(gui.plot) as ax:
             ax.plot(...)
     '''
+    from guietta.guietta_matplotlib import Colorbar
     try:
         ax = widget.ax
     except AttributeError as e:
         raise AttributeError('Class %s does not have an ''ax'' attribute.\n'
                       'Are you sure that you are referencing the correct plot?'
                        %  widget.__class__.__name__) from e
+        
+    # hack to remove all colorbars, in order to restore the ax geometry:
+    # creating a colorbar will resize the ax, and creating a new one
+    # will resize it further, unless the previous one has been removed.
+    for child in ax.get_children():
+        if isinstance(getattr(child, 'colorbar', None), Colorbar):
+            child.colorbar.remove()
+
     ax.clear()
 
     yield ax
@@ -1133,7 +1234,7 @@ class M(_DeferredCreationWidget):
     Creating an object of this class will import the matplotlib module.
     '''
     def __init__(self, name, width=5, height=3, dpi=100,
-                       subplots=(1,1), **kwargs):
+                       subplots=(1,1), animated=False, **kwargs):
 
         self._name = name
         self._width = width
@@ -1141,6 +1242,7 @@ class M(_DeferredCreationWidget):
         self._dpi = dpi
         self._subplots = subplots
         self._kwargs = kwargs
+        self._animated = animated
 
     def create(self, gui):
         from guietta import guietta_matplotlib
@@ -1149,10 +1251,22 @@ class M(_DeferredCreationWidget):
         _default_signals[widget_class] = 'clicked'
 
         widget = widget_class(self._width, self._height, self._dpi,
-                              self._subplots, **self._kwargs)
+                              self._subplots, self._animated, **self._kwargs)
         return (widget, self._name)
 
 
+class MA(M):
+    '''Animated Matplotlib widget
+
+    First display works as usual. When new data is assigned to the widget,
+    the set_ydata (for plots) or set_array (for images) method is called to
+    update the display without redrawing the entire widget.
+    '''
+    def __init__(self, name, width=5, height=3, dpi=100,
+                       subplots=(1,1), **kwargs):
+        super().__init__(name, width, height, dpi, subplots, animated=True, **kwargs)
+
+
 class PG(_DeferredCreationWidget):
     '''A pyqtgraph PlotWidget.
     
@@ -1190,9 +1304,14 @@ class PGI(_DeferredCreationWidget):
 # Stdout redirection
 
 class StdoutLog(QPlainTextEdit):
-    '''Log widget showing the stdout/stderr in the GUI'''
+    '''Log widget showing the stdout/stderr in the GUI
 
+    By default, stdout/stderr is redirected just before
+    setup() is called, while the original stdout/err is kept
+    while the GUI is initialized.
+    '''
     newData = Signal(str)
+    active = False
 
     def __init__(self):
         super().__init__('')
@@ -1202,6 +1321,7 @@ class StdoutLog(QPlainTextEdit):
         # because if the logging module is used, it makes a copy of
         # sys.stderr and the replacement would not work!
 
+        self._orig = sys.stdout.write, sys.stderr.write
         sys.stdout.write = self._write
         sys.stderr.write = self._write
 
@@ -1214,9 +1334,12 @@ class StdoutLog(QPlainTextEdit):
         self.newData.emit(data)
 
     def dataAvail(self, data):
-        text = data.strip()
-        if text != '':
-            self.appendPlainText(text)
+        if self.active:
+            text = data.strip()
+            if text != '':
+                self.appendPlainText(text)
+        else:
+            self._orig[0](data)
 
 
 # Some helper functions
@@ -1469,21 +1592,31 @@ def _convert_compacts(x):
     Compact elements processing.
 
     Converts:
-        '__xxx___' to QLineEdit('xxx')
+        '__xxx__:yy' to (QLineEdit('yy'), 'xxx')
+        '__xxx__' to (QLineEdit(''), 'xxx')
         'xxx'     to L('xxx')
         ['xxx']   to B('xxx')
         ['xxx', 'yyy']   to B('xxx', 'yyy')
+        callable  to button(x.__name__)
+        GUI object to G('name')
 
     L and B are used instead of QLabel and QPushButton in order to support
     images if xxx is a valid filename.
     Lists with zero length or longer than 2 elements raise a ValueError.
+    A callable is converted to a button with text set to the callable's name,
+    and the callable itself as the button slot.
     '''
 
-    if isinstance(x, str) and x.startswith('__') and x.endswith('__'):
-        return QLineEdit(x[2:-2])
+    if isinstance(x, str):
+        m = re.match(r'__(\w+)__\:(.*)', x)
+        if m:
+            return (QLineEdit(m.group(2)), m.group(1))
 
-    elif isinstance(x, str):
-        return L(x)
+        elif x.startswith('__') and x.endswith('__'):
+            return (QLineEdit(''), x[2:-2])
+
+        else:
+            return L(x)
 
     elif isinstance(x, list) and isinstance(x[0], str):
         if len(x) == 1 or len(x) == 2:
@@ -1491,6 +1624,12 @@ def _convert_compacts(x):
         else:
             raise ValueError('Invalid syntax: ' + str(x))
 
+    elif callable(x) and not isinstance(x, type) and hasattr(x, '__name__'):
+        return _AutoConnectButton(x.__name__.replace('_', ' '), x)
+
+    elif isinstance(x, Gui):
+        return SubGui(x)
+
     elif isinstance(x, tuple) and len(x) == 2:
         return (_convert_compacts(x[0]), x[1])
 
@@ -1498,6 +1637,19 @@ def _convert_compacts(x):
         return x  # No change
 
 
+class SubGui(_DeferredCreationWidget):
+    '''Groupbox holding a sub-gui'''
+
+    def __init__(self, subgui):
+        self.subgui = subgui
+
+    def create(self, gui):
+        groupbox = QGroupBox()
+        groupbox.setLayout(self.subgui.layout())
+        gui.add_as_subgui(self.subgui)
+        return groupbox
+
+
 ##################
 # GUIs persistence
 # The global list keeps references to all GUIs and allows them
@@ -1608,7 +1760,7 @@ def connect(widget, signal_name='default', slot=None):
     else:
         use_slot = functools.partial(slot, gui)
 
-    signal.connect(_exception_wrapper(use_slot, gui._exception_mode))
+    signal.connect(_exception_wrapper(use_slot, gui))
 
 
 class Gui:
@@ -1634,7 +1786,9 @@ class Gui:
                                persistence=PERSISTENT,
                                title='',
                                font=None,
-                               manage_threads=True):
+                               manage_threads=True,
+                               setup=None,
+                               use_formats=True):
 
         # This line must be the first one in this method otherwise
         # __setattr__ does not work.
@@ -1656,6 +1810,10 @@ class Gui:
         self._manage_threads = manage_threads
         self._main_thread = threading.get_ident()
 
+        self._timer = None
+        self._timer_count = 0
+        self._user_timer_callback = None
+
         self._get_handler = False   # These three for the get() method
         self._event_queue = queue.Queue()
         self._closed = False
@@ -1665,10 +1823,19 @@ class Gui:
 
         self.images_dir = images_dir
         self.is_running = False
+        self.use_formats = use_formats
 
-        # Prefedined button groups
+        # Predefined button groups
         self._groups = [QButtonGroup() for i in range(10)]
 
+        # Setup function
+        if setup and not callable(setup):
+            raise Exception('Setup value must be callable')
+
+        self._setup_func = setup
+        self._setup_done = False
+        self._subguis_to_setup = []
+
         # Input argument checks
         self._rows = Rows(lists)
         self._rows.map_in_place(_convert_compacts)
@@ -1729,17 +1896,26 @@ class Gui:
                 ContextMixIn.convert_object(widget)
                 self._layout.addWidget(widget, i, j, rowspan, colspan)
                 self._widgets[name] = widget
-
-                # Special case for QLineEdit, make it empty.
-                if isinstance(widget, QLineEdit):
-                    if not isinstance(widget, _QLineEditDoNotErase):
-                        widget.setText('')
-
                 done.add(element)
 
         self._align_guietta_properties()
         self.title(title)
 
+    def _setup(self):
+        '''
+        Call the setup function, if any, for this GUI and all
+        sub-GUIs. Also enable stdout/err redirection before setting up.
+        '''
+        StdoutLog.active = True
+
+        if not self._setup_done:
+            if self._setup_func:
+                _exception_wrapper(self._setup_func, self)(self)
+            for subgui in self._subguis_to_setup:
+                subgui._setup()
+
+        self._setup_done = True
+
     def _align_guietta_properties(self):
         '''Make sure that any and all widgets have a property'''
 
@@ -1778,7 +1954,7 @@ class Gui:
         return self._original_names
 
     def proxy(self, name):
-        '''Returns the *guietta property* for the a (normalized) widget name.
+        '''Returns the *guietta property* for a (normalized) widget name.
 
         A guietta property is an instance of the *GuiettaProperty* class,
         with two attributes: get() and set()
@@ -1853,7 +2029,7 @@ class Gui:
         rows = Rows(lists)
         rows.check_same(self._rows, allow_less_rows=True)
 
-        for i, j, stretch in rows.enumerate():
+        for i, _, stretch in rows.enumerate():
             self._layout.setRowStretch(i, stretch)
 
     def column_stretch(self, *lists):
@@ -1870,7 +2046,7 @@ class Gui:
         rows = Rows(lists)
         rows.check_same(self._rows, allow_less_rows=True)
 
-        for i, j, stretch in rows.enumerate():
+        for _, j, stretch in rows.enumerate():
             self._layout.setColumnStretch(j, stretch)
 
     def events(self, *lists):
@@ -1961,6 +2137,37 @@ class Gui:
 
         self._align_guietta_properties()
 
+    def timer_start(self, callback, interval=1.0):
+        '''Set up a timer to call *callback* every *interval* seconds.
+        
+        The callback will receive the Gui instance as its only argument.
+        '''
+        self._timer = QTimer()
+        self._user_timer_callback = _exception_wrapper(callback, self)
+        self._timer.timeout.connect(self._timer_callback)
+        self._timer.start(interval*1000)
+
+    def timer_stop(self):
+        '''Stops the timer'''
+        if self._timer is not None:
+            self._timer.stop()
+
+    def timer_count(self):
+        '''Returns the number of times the timer has been fired'''
+        return self._timer_count
+
+    def _timer_callback(self):
+        self._timer_count += 1
+        if self._user_timer_callback is not None:
+            self._user_timer_callback(self)
+
+    def timer(self, interval):
+        '''Decorator that starts a function using the GUI timer'''
+        def decorator(func):
+            self.timer_start(func, interval)
+            return func
+        return decorator
+        
     def __getitem__(self, name):
         '''Widget by coordinates [row,col]'''
         return self._layout.itemAtPosition(name[0], name[1]).widget()
@@ -2009,6 +2216,7 @@ class Gui:
         This call is blocking and will return when the window is closed.
         Any user interaction must be done with callbacks.
         '''
+        self._setup()
         self.show()
         self.is_running = True
         self._app.exec_()
@@ -2076,6 +2284,7 @@ class Gui:
             self._get_handler = True
 
         self.window().closeEvent = self._stop_handler
+        self._setup()
         self.show()
         self._closed = False
 
@@ -2205,6 +2414,15 @@ class Gui:
 
         return func
 
+    def add_as_subgui(self, subgui):
+        '''
+        Set *subgui* as a sub-gui of this one. Exception modes will be
+        propagated, and the sub-gui setup function will be called
+        just after the main gui one.
+        '''
+        subgui._exception_mode = self._exception_mode
+        self._subguis_to_setup.append(subgui)
+
 
 class GuiIterator():
     '''An iterator that allows looping over the GUI events:
